#! /usr/bin/env node

var fs = require('fs');
var path = require("path");
var S = require('string');
var _ = require('underscore');
var javadirfilter = ""

/* Globals */


var ocurrences = new Array(); //donde se guardan las ocurrencias de cadenas en cada archivo
var root = process.cwd();


//TODO: volver eso un parametro que indique la carpeta donde buscar, o carpetas?, o usar archivo de config
var srcpath = path.join(root,"src/com/mind/visualsales/activities/");
var respath = path.join(root,"res");
var layoutspath = path.join(respath,"layout");

var walk = function(dir, done,filter) {
  var results = [];
  fs.readdir(dir, function(err, list) {
    if (err) return done(err);
    var pending = list.length;
    if (!pending) return done(null, results);
    list.forEach(function(file) {
      file = path.join(dir,file);
      fs.stat(file, function(err, stat) {
        if (stat && stat.isDirectory()) {
          walk(file, function(err, res) {
            results = results.concat(res);
            if (!--pending) done(null, results);
          },filter);
        } else {
          if(filter)
          {
             
              if(filter(file)){
                results.push(file);
              }
          }
          else{
           
            //no filter defined, add all files
            results.push(file);

          }

          
          if (!--pending) done(null, results);
        }
      });
    });
  });
};



//Generates index from files with the ocurrences of all the strings.
//returns array of objects with the following structure
/*
  {
   path:"filepath",
   string:"string found here",
   line:23,
   row:10,

   }
*/
var processJavaFiles = function(files,callback){
  var count = files.length;

  files.forEach(function(file){
     //Buscar todas las ocurrencias de cadenas
     
     fs.readFile(file,"utf-8",function(err,data){
      console.log("----processing file:-----"+file);
      if(err){
        console.log("** ERROR ** ->"+err);
        return;
      }
      else{
        //Obtain all string ocurrences
        var regex=/"(?:[^"\\]+|\\.)*"/;

        var lines = data.split("\n");
          var linenum = 1;
        //console.log("lines:"+lines.length);
        lines.forEach(function(line){
            var matches = regex.exec(line);
            if(matches != null)
            {
              matches.forEach(function(match){
                //excluir cadenas que sean parte de un llamado a .log o sean vacias
                if(validMatch(line,match))
                { 
                  //ya tenemos el nombre del recurso , creamos la ocurrencia y la agregamos a la lista de ocurrencias
                  var ocurrence = new StringOcurrence({str:match,"path":file});
                  ocurrence.ocurrenceType = "java";
                  ocurrence.lineNumber = linenum;
                  ocurrence.context = line;
                  ocurrences.push(ocurrence);

                }
                
              });
            }
            linenum++;
        });
      }
      console.log("end processing file");
      count--;
      if(count == 0){
        //Terminó de procesar todos los archivos y cargo la lista de ocurrencias.
        if(callback)callback();
      }
     });
     
  });
};

/* Aplica una serie de filtros como
que no sea parte de un log statement, 
que no sea cadena vacia,
*/
var validMatch = function(line,match){

 var logs = ["Log.v(","Log.d(","Log.e(","Log.w(","Log.wtf(","Log.println(","Log.i("];
 if(_.any(logs,function(logid){return line.indexOf(logid)>=0})){
      return false;
   }

  if(match == null || S(match).trim().s.length == 0 || S(match).replaceAll('"','').trim().s.length == 0)
      return false;

  if(match.indexOf("@style/")>-1 || match.indexOf("@color/")>-1)
    return false;

  return true;
}

var processLayoutFiles = function(files,callback){
  var count = files.length;
  var regexLiteral=/"(?:[^"\\]+|\\.)*"/; //regex para sacar solo lo que esta entre comillas
  files.forEach(function(file){
     //Buscar todas las ocurrencias de cadenas
     
     fs.readFile(file,"utf-8",function(err,data){
      console.log("----processing file:-----"+file);
      if(err){
        console.log("** ERROR ** ->"+err);
        return;
      }
      else{
        //Obtain all string ocurrences
        /* Esta expresion regular hace match con text = "algo" tooltipText = "algo"
           necesitamos capturar el fragmento text =  para evitar que por accidente
           se procesen cosas que no sean cadenas.
        */
        var regex=/[t|T]ext[\s]*=[\s]*"(?:[^"\\]+|\\.)*"/;
        var lines = data.split("\n");
  
        var linenum = 1;
        //console.log("lines:"+lines.length);
        lines.forEach(function(line){
            var matches = regex.exec(line);

            //si la cadena no apunta a un recurso de android
            if(line.indexOf("@string/")<0 && matches != null)
            {
               matches.forEach(function(match){
               
               if(validMatch(line,match)){ 
                 //TODO: Deberia haber solo un literal, por eso asumimos que el tama es uno
                  var literal = regexLiteral.exec(match)[0];
                  //ya tenemos el nombre del recurso , creamos la ocurrencia y la agregamos a la lista de ocurrencias
                  var ocurrence = new StringOcurrence();
                  ocurrence.path = file;
                  ocurrence.str = literal;
                  ocurrence.fragment = match;
                  ocurrence.ocurrenceType = "layout";
                  ocurrence.lineNumber = linenum;
                  ocurrence.context = line;
                  ocurrences.push(ocurrence);
                }
                
                //
              });
            }
            linenum++;
        });
      }
      console.log("end processing file");
      count--;
      if(count == 0){
        //Terminó de procesar todos los archivos y cargo la lista de ocurrencias.
        if(callback)callback();
      }
     });
     
  });
};





var StringOcurrence = function(defaults){
  defaults = defaults || {};
  this.str = defaults.str || "";
  this.path = defaults.path || "";
  this.lineNumber = defaults.lineNumber || -1;
  this.ocurrenceType = defaults.ocurrenceType;

  this.context = defaults.context || ""; //el contexto me da la linea del codigo fuente 
    //flag para el proceso de actualizacion de strings que indica si se debe tener en cuenta la ocurrencia
  this.include = defaults.include=== undefined?true:defaults.include;
}


StringOcurrence.prototype.toString = function(){
  return "str:"+this.str+" resourceName:"+this.resourceName+" path:"+this.path;
}


/**
  Crea el archivo string_ocurrences.log que será usado como paso intermedio antes
  de modificar los archivos fuentes, permite cambiar el nombre de los recursos antes.
*/
var generateOcurrencesLogFile = function(ocurrences){
  var info ={'ocurrences':ocurrences};
  var json = JSON.stringify(info,null,4); //Generate json file  formatted for easy editing
  var logpath = path.join(root,"string_ocurrences.log");
  fs.writeFile(logpath,json,
    function(err){
      if(err){throw err;}
      console.log("===> Temporary log file generated, use androidstr-update to apply changes <======",logpath);

   }
  );
}





//======Process .java files==============//

var parseJavaFiles = function(callback){

  console.log("srcpath:",srcpath);
    walk(srcpath,function(err,res){
      if(err) console.log("ERROR:",err);
      //Process .java files
      console.log("resultado walk:",res);
      processJavaFiles(res,callback);

    },
    function(file){
      //filter: only java files
        return path.extname(file) === ".java";
    });

};

var parseLayoutFiles = function(callback){

    //Ahora procesar los layouts
    walk(respath,function(err,res){
      processLayoutFiles(res,callback);
    },function(file){
      //filter: only layout files
      //we test that the directory name contains the word layout, so layout-land and other variations qualify
        return path.extname(file) === ".xml" && path.dirname(file).indexOf("layout")>=0;
    });
  
};


parseJavaFiles(function(){
  parseLayoutFiles(function(){
    //Con la coleccion de ocurrencies generar el archivo de log
    generateOcurrencesLogFile(ocurrences);
  })
})




