#! /usr/bin/env node
/*
This module processes a previously generated log file with the strings and
generates strings.xml and modifies the source code files to reference those strings.
No imports are added (you have to manually do that)
*/

var fs = require('fs');
var path = require('path');
var Mustache = require('mustache');
var xml2js = require('xml2js');
var mkdirp = require('mkdirp');
var _ = require('underscore');


var stringFileTemplatePath = path.join(__dirname,"assets/strings.xml.mustache");
var logfilename = 'string_ocurrences.log';

var root = process.cwd();
var srcpath = path.join(root,"src");
var respath = path.join(root,"res");
var valuespath = path.join(respath,"values");
var stringsFilePath = path.join(root,"res/values/strings.xml");
var resourceStrings = new Array();

//TODO: parametrizar este nombre, debe ser una utilidad que retorne el recurso y debe estar en la clase global. Dar instrucciones para usarlo
var getResourceStaticMethod="com.mind.visualsales.app.VisualSaleApplication.getStringRes";


/* 
 * Intenta cargar el archivo strings.xml y cargar los recursos definidos
 */
var loadExistingStrings = function(callback){

  var parser,stringsxmlcontent
  if(!fs.existsSync(stringsFilePath))
  {
    return;
  }

  parser = new xml2js.Parser();
  stringsxmlcontent = fs.readFileSync(stringsFilePath,"utf-8");
   parser.parseString(stringsxmlcontent,function(err,result){
      var strings = result.resources.string;
      if(strings){
        strings.forEach(function(stringItem){
            var stringText = stringItem["_"];
            var stringName = stringItem["$"].name;
            //Agregar recurso indicando que existia previamente en strings.xml
            addStringResource(stringName,stringText,true);
        });
      }
      if(callback)callback();
   });

}

/* Process the log file and modify all the string ocurrences
   and save strings.xml
   */


  var processProject = function(callback){

  	var logpath = path.join(root,logfilename);
  	fs.readFile(logpath,function(err,data){
  		if(err){
  			console.log('Error reading log file, please verify the path:',logpath);
  			return ;
  		} 
  		var infolog = JSON.parse(data);
  		var ocurrences = infolog.ocurrences;
      var processedOcurrences = 0;
  		ocurrences.forEach(function(ocurrence){
        if(ocurrence.include == false)
          return;

  			if(ocurrence.ocurrenceType == "java"){

            processJavaOcurrence(ocurrence);
        }
        else{

            processLayoutOcurrence(ocurrence);
        }

        processedOcurrences++;

  		});

      console.log("Processed Ocurrences:",processedOcurrences);
  		if(callback)callback();
  	});
  };

var processJavaOcurrence = function(ocurrence){
      //Para cada ocurrencia modificar el archivo donde se encuentra reemplazando la cadena
      //por su correspondiente R.string.identificador

      //Por ahora hacerlo sincronicamente porque sino tendriamos problemas si se modifica el archivo en paralelo
      var filedata = fs.readFileSync(ocurrence.path,"utf-8");
       var resourceName = generateValidResourceName(ocurrence.str);
       //generar el llamado al metodo que debe existir en la clase global del proyecto
       var resourceRef = getResourceStaticMethod+"(R.string."+resourceName +")";
      //var updated = filedata.replace(reg,resourceref);
      //usamos nuestra propia version de replace que no usa regex
      var updated = replaceString(filedata,ocurrence.str,resourceRef);
      fs.writeFileSync(ocurrence.path,updated,"utf-8");
      console.log("processed string in file:",ocurrence.path);
      //Agregar string a diccionario
      addStringResource(resourceName,ocurrence.str,false);
}


//reemplaza un bloque de texto por otro dentro de una cadena, solo reemplaza la primera ocurrencia
var replaceString = function(str,part1,part2){

   var start = str.indexOf(part1);
   var len = part1.length;
   var endstring = start+len;
  return str.substring(0,start) + part2+str.substring(endstring);
}

//Similar a processJavaOcurrence pero tiene en cuenta la sintaxis para layouts
/* Las ocurrencias en layouts son del tipo text = "este es el texto"
   deben ser reemplazadas por text = "@string/"
   Recordar que esto puede aplicar a declaraciones de tipo tooltipText = "esta es la cadena"
   la expresion usada para capturar estas cadenas fue

    Tener en cuenta que para las cadenas de layout la ocurrencia tiene dos propiedades
    str: guarda la ocurrencia con comillas
    fragment: guarda la asigaion de tipo tooltipText = "esta es la cadena"

    var regex=/[t|T]ext[\s]*=[\s]*"(?:[^"\\]+|\\.)*"/;
*/
var processLayoutOcurrence = function(ocurrence){

      var filedata = fs.readFileSync(ocurrence.path,"utf-8");
      //Generamos el nombre del recurso para strings.xml teniendo en cuenta que no haya conflicto con recursos existentes en strings.xml
      var resourceName=generateValidResourceName(ocurrence.str);
      var resourceref = "\"@string/"+resourceName+"\"";

      var start = filedata.indexOf(ocurrence.str);
      var len = ocurrence.str.length;
      var endstring = start+len;


      //debemos reemplazar el fragmento por la versión con el literal actualizado
      //TODO: mejorar esto, seria mejor leer a un array el archivo y usar los numeros de lineas o algo asi
      // esto implicaria procesar en batch todas las ocurrencias del mismo archivo.
      var updatedFragment = replaceString(ocurrence.fragment,ocurrence.str,resourceref);
      var updated = replaceString(filedata,ocurrence.fragment,updatedFragment);
      fs.writeFileSync(ocurrence.path,updated,"utf-8");
      console.log("processed string in file:",ocurrence.path,"-",updatedFragment);
      //Agregar string a diccionario
      addStringResource(resourceName,ocurrence.str,false);
}

var addedResources = {}; //hash para no duplicar recursos
var addStringResource = function(key,val,existing){

  var  tempkey = "_"+key; //en caso de que la cadena sea espacio vacio
  if(addedResources[tempkey]) //no agregar el recurso si ya fue agregado uno con el mismo identificador
    return;

  console.log("addStringResource:",key,"-",val);
  resourceStrings.push({"name":key,"string":val,"existing":existing?true:false});
  addedResources[tempkey] = true;
}


var cacheResources={}; //hash donde la llave es la CADENA y el valor es el identificador (al revez)
/* Intenta generar un nombre de recurso a partir de la cadena suministrada
pero si no puede usa el nombre generico string_n  */
var generateValidResourceName = function(str){

  //TODO: debe haber una opcion para no reutilizar la misma cadena, por ahora siempre se reutilizara el mismo recurso
  if(cacheResources[str])
    return cacheResources[str];

  //TODO: Agregar opcion que permita generar el nombre a partir de cadena, ver como lo hace adt tools
  //contador usado para generar los nombres de recursos
  if(generateValidResourceName.counter)
      generateValidResourceName.counter++;
  else
  {
     generateValidResourceName.counter = 1;
  }

  var newresourcename = "string_"+generateValidResourceName.counter;
  //si ya existe un recurso con dicho nombre seguir iterando hasta que no exista ninguno
  while(resourceNameExists(newresourcename)){
    newresourcename = generateValidResourceName();
  }
  //actualizar cache
  cacheResources[str] = newresourcename;
  return newresourcename;
}

var resourceNameExists = function(name){
 return _.any(resourceStrings,function(item){
    return item.name === name;
 });
}

//Genera el archivo strings.xml a partir
var saveStringsXmlFile = function(path,callback){

  //Necesitamos crear el path en caso de que no exista res/values/
  //mkdirp crea recursivamente el directorio
  mkdirp(valuespath,function(err){
    if(err)console.error(err);


    var filetemplate = fs.readFileSync(stringFileTemplatePath,"utf-8");
    var filecontent = generateStringsFileContent();
    var content = Mustache.render(filetemplate,{CONTENT:filecontent});

    fs.writeFileSync(path,content,"utf-8");
    console.log("** File strings.xml saved:",path,"***");
    if(callback)callback(content);

  });
}

//Retorna una cadena con los nodos xml que componen el archivo strings.xml
var generateStringsFileContent = function(){
  var content = new Array();
  var templatecdata = "<string name=\"{{STRINGNAME}}\"><![CDATA[{{{STRINGVAL}}}]]></string>";
  var templatenormal= "<string name=\"{{STRINGNAME}}\">{{{STRINGVAL}}}</string>";

  var actualstring;
  resourceStrings.forEach(function(item){
    console.dir(item);
    //debemos quitar la primera y ultima doble comilla
    actualstring = item.string;
    //solo para las cadenas extraidas no existentes previamente debemos quitar las comillas iniciales y finales
    if(!item.existing)
      actualstring = item.string.substring(1,item.string.length-1);

    if(actualstring === undefined){
      console.log("** DEBUG: actualstring is null!!!! -->")
      return;
    }
    //escape string for xml
    actualstring = actualstring.replace(/&/g, '&amp;')
               .replace(/</g, '&lt;')
               .replace(/>/g, '&gt;')
               .replace(/"/g, '&quot;');

    var stringnode = Mustache.render(templatenormal,{STRINGNAME:item.name,STRINGVAL:actualstring});
    content[content.length] = stringnode+"\n";
  });
  
  return content.join('');
}

console.log("** Ready to update android project **");
  //cargar strings.xml existentes
  loadExistingStrings(function(){
      processProject(function(){
        saveStringsXmlFile(stringsFilePath,function(content){
          console.log("string xml content:",content);
        });
      });
  });
  
